"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tracing_1 = require("./tracing");
const path = require("path");
const fs = require("fs");
const browsers_1 = require("@puppeteer/browsers");
const grpc_plugin_1 = require("./plugin/v2/grpc_plugin");
const http_server_1 = require("./service/http-server");
const logger_1 = require("./logger");
const minimist = require("minimist");
const node_plugin_1 = require("./node-plugin");
const Sanitizer_1 = require("./sanitizer/Sanitizer");
const config_1 = require("./config/config");
async function main() {
    const argv = minimist(process.argv.slice(2));
    const env = Object.assign({}, process.env);
    const command = argv._[0];
    if (command === undefined) {
        const config = (0, config_1.getConfig)();
        const logger = new logger_1.PluginLogger();
        if (config.rendering.tracing.url) {
            (0, tracing_1.startTracing)(logger);
        }
        if (!config.rendering.chromeBin && process.pkg) {
            const execPath = path.dirname(process.execPath);
            const chromeInfoFile = fs.readFileSync(path.resolve(execPath, 'chrome-info.json'), 'utf8');
            const chromeInfo = JSON.parse(chromeInfoFile);
            config.rendering.chromeBin = (0, browsers_1.computeExecutablePath)({
                cacheDir: path.dirname(process.execPath),
                browser: browsers_1.Browser.CHROMEHEADLESSSHELL,
                buildId: chromeInfo.buildId,
            });
            logger.debug(`Setting chromeBin to ${config.rendering.chromeBin}`);
        }
        await (0, node_plugin_1.serve)({
            handshakeConfig: {
                protocolVersion: 2,
                magicCookieKey: 'grafana_plugin_type',
                magicCookieValue: 'datasource',
            },
            versionedPlugins: {
                2: {
                    renderer: new grpc_plugin_1.RenderGRPCPluginV2(config, logger),
                },
            },
            logger: logger,
            grpcHost: config.plugin.grpc.host,
            grpcPort: config.plugin.grpc.port,
        });
    }
    else if (command === 'server') {
        const config = (0, config_1.getConfig)();
        const logger = new logger_1.ConsoleLogger(config.service.logging);
        if (config.rendering.tracing.url) {
            (0, tracing_1.startTracing)(logger);
        }
        const sanitizer = (0, Sanitizer_1.createSanitizer)();
        const server = new http_server_1.HttpServer(config, logger, sanitizer);
        await server.start();
    }
    else {
        console.log('Unknown command');
    }
}
main().catch((err) => {
    const errorLog = {
        '@level': 'error',
        '@message': 'failed to start grafana-image-renderer',
        error: err.message,
        trace: err.stack,
    };
    console.error(JSON.stringify(errorLog));
    process.exit(1);
});
//# sourceMappingURL=app.js.map