"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConfig = void 0;
const fs = require("fs");
const _ = require("lodash");
const minimist = require("minimist");
const config_1 = require("../service/config");
const config_2 = require("../plugin/v2/config");
function getConfig() {
    const argv = minimist(process.argv.slice(2));
    const env = Object.assign({}, process.env);
    const command = argv._[0];
    if (command === 'server') {
        let config = config_1.defaultServiceConfig;
        if (argv.config) {
            try {
                const fileConfig = readJSONFileSync(argv.config);
                config = _.merge(config, fileConfig);
            }
            catch (e) {
                console.error('failed to read config from path', argv.config, 'error', e);
            }
        }
        (0, config_1.populateServiceConfigFromEnv)(config, env);
        return config;
    }
    const config = config_2.defaultPluginConfig;
    (0, config_2.populatePluginConfigFromEnv)(config, env);
    return config;
}
exports.getConfig = getConfig;
function readJSONFileSync(filePath) {
    const rawdata = fs.readFileSync(filePath, 'utf8');
    return JSON.parse(rawdata);
}
//# sourceMappingURL=config.js.map