"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupRateLimiter = void 0;
const rate_limiter_flexible_1 = require("rate-limiter-flexible");
const ioredis_1 = require("ioredis");
const setupRateLimiter = (config, log) => {
    let rateLimiter;
    if (config.redisHost && config.redisPort) {
        const redisClient = new ioredis_1.Redis({
            host: config.redisHost,
            port: config.redisPort,
        });
        rateLimiter = new rate_limiter_flexible_1.RateLimiterRedis({
            storeClient: redisClient,
            keyPrefix: 'rate-limit',
            points: config.requestsPerSecond,
            duration: 1, // per second
        });
        log.info('Rate limiter enabled using Redis', 'requestsPerSecond', config.requestsPerSecond);
    }
    else {
        // Fallback to in-memory storage
        rateLimiter = new rate_limiter_flexible_1.RateLimiterMemory({
            points: config.requestsPerSecond,
            duration: 1,
        });
        log.info('Rate limiter enabled using in-memory storage', 'requestsPerSecond', config.requestsPerSecond);
    }
    return rateLimiter;
};
exports.setupRateLimiter = setupRateLimiter;
//# sourceMappingURL=ratelimiter.js.map