"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startTracing = void 0;
const sdk_node_1 = require("@opentelemetry/sdk-node");
const auto_instrumentations_node_1 = require("@opentelemetry/auto-instrumentations-node");
const exporter_trace_otlp_http_1 = require("@opentelemetry/exporter-trace-otlp-http");
const semantic_conventions_1 = require("@opentelemetry/semantic-conventions");
const resources_1 = require("@opentelemetry/resources");
const api_1 = require("@opentelemetry/api");
const config_1 = require("./config/config");
const config = (0, config_1.getConfig)();
let sdk;
if (config.rendering.tracing.url) {
    sdk = initTracing(config.rendering.tracing.url, config.rendering.verboseLogging);
}
function initTracing(exporterURL, verboseLogging = false) {
    if (verboseLogging) {
        api_1.diag.setLogger(new api_1.DiagConsoleLogger(), api_1.DiagLogLevel.DEBUG);
    }
    const traceExporter = new exporter_trace_otlp_http_1.OTLPTraceExporter({
        url: exporterURL,
    });
    return new sdk_node_1.NodeSDK({
        resource: new resources_1.Resource({
            [semantic_conventions_1.ATTR_SERVICE_NAME]: config.rendering.tracing.serviceName || 'grafana-image-renderer',
        }),
        traceExporter,
        instrumentations: [
            (0, auto_instrumentations_node_1.getNodeAutoInstrumentations)({
                // only instrument fs if it is part of another trace
                '@opentelemetry/instrumentation-fs': {
                    requireParentSpan: true,
                },
            }),
        ],
    });
}
function startTracing(log) {
    sdk.start();
    log.info('Starting tracing');
    process.on('SIGTERM', () => {
        sdk
            .shutdown()
            .then(() => log.debug('Tracing terminated'))
            .catch((error) => log.error('Error terminating tracing', 'err', error))
            .finally(() => process.exit(0));
    });
}
exports.startTracing = startTracing;
//# sourceMappingURL=tracing.js.map